# Hydrological Modelling in R
# This script contains five individual hydrological models for the five 
# sub-catchments of the Barotse Floodplain. 
# It takes inputs of precipitation and temperature (one CSV, formatted as
# three columns: Timestamp, Precipitation, and Temperature)

# RUN TIME - 2 mins

library(hydromad) # for hydrological modelling
library(zoo)      # for working with time series data

# Define catchment areas (in square kilometres)
# UZB = 93090 # Upper Zambezi
# KAB = 71339 # Kabompo
# LUA = 32825 # Luanginga
# LUE = 15320 # Luena
# LUG = 40587 # Lugwenbungu


# Specify directories for input data (precipitation and temperature, merged as one CSV as columns: [1] Timestamp [2] Precipitation [3] Temperature)

# folder for UPPER ZAMBEZI files
UZB_files <- dir(path = "D:\\3b_pr_tas_merged\\UZB", pattern = ".csv" )
UZB_folder <- "D:\\3b_pr_tas_merged\\UZB"

# folder for KABOMPO files
KAB_files <- dir(path = "D:\\3b_pr_tas_merged\\KAB", pattern = ".csv" )
KAB_folder <- "D:\\3b_pr_tas_merged\\KAB"

# folder for LUGWEBUNGU files
LUG_files <- dir(path = "D:\\3b_pr_tas_merged\\LUG", pattern = ".csv" )
LUG_folder <- "D:\\3b_pr_tas_merged\\LUG"

# folder for LUANGINGA files
LUA_files <- dir(path = "D:\\3b_pr_tas_merged\\LUA", pattern = ".csv" )
LUA_folder <- "D:\\3b_pr_tas_merged\\LUA"

# folder for LUENA files
LUE_files <- dir(path = "D:\\3b_pr_tas_merged\\LUE", pattern = ".csv" )
LUE_folder <- "D:\\3b_pr_tas_merged\\LUE"



# calibration for UPPER ZAMBEZI
uzb <- read.csv("C:\\Users\\gy16e3m\\OneDrive - University of Leeds\\PhD\\Chapter_1\\Hydrological modelling\\3a_pr_tas_q_merged\\3a_pr_tas_q_Zambezi_catchment.csv")
uzb$Date <- as.Date(uzb$Date, format = "%d/%m/%Y")
uzb.zoo <- as.zoo(uzb)
uzb.zoo <- read.zoo(uzb)
uzb_start = as.Date("2005-10-01")
uzb_end = as.Date("2010-02-25")
uzb_calib = uzb[uzb$Date >= uzb_start & uzb$Date <= uzb_end,]
uzb_calib.zoo = as.zoo(uzb_calib)
uzb_calib.zoo = read.zoo(uzb_calib)
uzb_calib.zoo$Q <- convertFlow(uzb_calib.zoo$Q, from = "cumecs", to = "mm/day", 
                               area = 93090)

# calibration for KABOMPO
kab <- read.csv("C:\\Users\\gy16e3m\\OneDrive - University of Leeds\\PhD\\Chapter_1\\Hydrological modelling\\3a_pr_tas_q_merged\\3a_pr_tas_q_Kabompo_catchment.csv")
kab$Date <- as.Date(kab$Date, format = "%d/%m/%Y")
kab.zoo <- as.zoo(kab)
kab.zoo <- read.zoo(kab)
kab_start = as.Date("2005-10-01")
kab_end = as.Date("2010-02-25")
kab_calib = kab[kab$Date >= kab_start & kab$Date <= kab_end,]
kab_calib.zoo = as.zoo(kab_calib)
kab_calib.zoo = read.zoo(kab_calib)
kab_calib.zoo$Q <- convertFlow(kab_calib.zoo$Q, from = "cumecs", to = "mm/day", 
                               area = 71339)

# calibration for LUGWEBUNGU
lug <- read.csv("C:\\Users\\gy16e3m\\Documents\\HYDROMAD\\lungwebungu\\3a_pr_tas_q_Lugwebungu_catchment_AVERAGED_MINIMA.csv")
lug$Date <- as.Date(lug$Date, format = "%d/%m/%Y")
lug.zoo <- as.zoo(lug)
lug.zoo <- read.zoo(lug)
lug_start = as.Date("1980-10-01")
lug_end = as.Date("1985-02-25")
lug_calib = lug[lug$Date >= lug_start & lug$Date <= lug_end,]
lug_calib.zoo = as.zoo(lug_calib)
lug_calib.zoo = read.zoo(lug_calib)
lug_calib.zoo$Q <- convertFlow(lug_calib.zoo$Q, from = "cumecs", to = "mm/day", 
                               area = 40587)

# calibration for LUANGINGA
lua <- read.csv("C:\\Users\\gy16e3m\\OneDrive - University of Leeds\\PhD\\Chapter_1\\Hydrological modelling\\3a_pr_tas_q_merged\\3a_pr_tas_q_Luanginga_catchment.csv")
lua$Date <- as.Date(lua$Date, format = "%d/%m/%Y")
lua.zoo <- as.zoo(lua)
lua.zoo <- read.zoo(lua)
lua_start = as.Date("2006-10-01")
lua_end = as.Date("2010-09-30")
lua_calib = lua[lua$Date >= lua_start & lua$Date <= lua_end,]
lua_calib.zoo = as.zoo(lua_calib)
lua_calib.zoo = read.zoo(lua_calib)
lua_calib.zoo$Q <- convertFlow(lua_calib.zoo$Q, from = "cumecs", to = "mm/day", 
                               area = 32825)

# calibration for LUENA
lue <- read.csv("C:\\Users\\gy16e3m\\OneDrive - University of Leeds\\PhD\\Chapter_1\\Hydrological modelling\\3a_pr_tas_q_merged\\spinup_luena_2017_2018_three.csv")
lue$Date <- as.Date(lue$Date, format = "%d/%m/%Y")
lue.zoo <- as.zoo(lue)
lue.zoo <- read.zoo(lue)
lue.zoo$Q <- convertFlow(lue.zoo$Q, from = "cumecs", to = "mm/day", 
                         area = 15320)

# Check calibration datasets visually
#xyplot(kab_calib.zoo)
#xyplot(uzb_calib.zoo)
#xyplot(lua_calib.zoo)
#xyplot(lue.zoo)
#xyplot(lug_calib.zoo)



# Define hydrological models for each sub-catchment using the specified parameters 
UZB_mod = hydromad(uzb_calib.zoo, sma = "cmd", routing = "expuh", tau_s = 53.445, v_s = 1,
                   f = 0.646006, e = 0.275934, d = 389.333)

KAB_mod = hydromad(kab_calib.zoo, sma = "cmd", routing = "expuh", tau_s = 63.6958, v_s = 0.999757,
                   f = 0.476065, e = 1.5, d = 419.197)

LUG_mod = hydromad(lug_calib.zoo, sma = "cmd", routing = "expuh", tau_s = 99.9999, v_s = 1,
                   f = 0.478978, e = 1.5, d = 50.0004)

LUA_mod = hydromad(lua_calib.zoo, sma = "cmd", routing = "expuh", tau_s = 62.0885, v_s = 1,
                   f = 0.468852, e = 1.49799, d = 452.721)

LUE_mod = hydromad(lue.zoo, sma = "cmd", routing = "expuh", tau_s = 46.0722, v_s = 1,
                   f = 1.15598, e = 0.257299, d = 174.101)


# predict flow for UPPER ZAMBEZI
for(f in UZB_files){
  path = paste(UZB_folder, "\\", f, sep="")
  print(path)
  
  # create zoo object and read prediction data
  predict <- read.csv(path)
  colnames(predict)[2] <- "P"
  colnames(predict)[3] <- "E"
  predict.zoo <- as.zoo(predict)
  predict.zoo <- read.zoo(predict, format = "%d/%m/%Y")
  
  # generate model output
  output <- predict(UZB_mod, predict.zoo)
  
  # file path creator
  csv_path <- file.path("C:", "Users", "gy16e3m", "Downloads", paste(f,".csv", sep=""))
  
  # convert units to cumecs
  flows <- convertFlow(zoo(output), from = "mm/day", to =
                         "cumecs", area = 93089)
  
  # write to csv
  write.zoo(flows, file=csv_path, row.names=FALSE, col.names=TRUE, sep=",")
}


# predict flow for KABOMPO
for(f in KAB_files){
  path = paste(KAB_folder, "\\", f, sep="")
  print(path)
  
  # create zoo object and read prediction data
  predict <- read.csv(path)
  colnames(predict)[2] <- "P"
  colnames(predict)[3] <- "E"
  predict.zoo <- as.zoo(predict)
  predict.zoo <- read.zoo(predict, format = "%d/%m/%Y")
  
  # model
  output <- predict(KAB_mod, predict.zoo)
  
  # file path creator
  csv_path <- file.path("C:", "Users", "gy16e3m", "Downloads", paste(f,".csv", sep=""))
  
  # convert units to cumecs
  flows <- convertFlow(zoo(output), from = "mm/day", to =
                         "cumecs", area = 71339)
  
  # write to csv
  write.zoo(flows, file=csv_path, row.names=FALSE, col.names=TRUE, sep=",")
}



# predict flow for LUGWEBUNGU
for(f in LUG_files){
  path = paste(LUG_folder, "\\", f, sep="")
  print(path)
  
  # create zoo object and read prediction data
  predict <- read.csv(path)
  colnames(predict)[2] <- "P"
  colnames(predict)[3] <- "E"
  predict.zoo <- as.zoo(predict)
  predict.zoo <- read.zoo(predict, format = "%d/%m/%Y")
  
  # model
  output <- predict(LUG_mod, predict.zoo)
  
  # file path creator
  csv_path <- file.path("C:", "Users", "gy16e3m", "Downloads", paste(f,".csv", sep=""))
  
  # convert units to cumecs
  flows <- convertFlow(zoo(output), from = "mm/day", to =
                         "cumecs", area = 40587)
  
  # write to csv
  write.zoo(flows, file=csv_path, row.names=FALSE, col.names=TRUE, sep=",")
}



# predict flow for LUANGINGA
for(f in LUA_files){
  path = paste(LUA_folder, "\\", f, sep="")
  print(path)
  
  # create zoo object and read prediction data
  predict <- read.csv(path)
  colnames(predict)[2] <- "P"
  colnames(predict)[3] <- "E"
  predict.zoo <- as.zoo(predict)
  predict.zoo <- read.zoo(predict, format = "%d/%m/%Y")
  
  # model
  output <- predict(LUA_mod, predict.zoo)
  
  # file path creator
  csv_path <- file.path("C:", "Users", "gy16e3m", "Downloads", paste(f,".csv", sep=""))
  
  # convert units to cumecs
  flows <- convertFlow(zoo(output), from = "mm/day", to =
                         "cumecs", area = 32825)
  
  # write to csv
  write.zoo(flows, file=csv_path, row.names=FALSE, col.names=TRUE, sep=",")
}



# predict flow for LUENA
for(f in LUE_files){
  path = paste(LUE_folder, "\\", f, sep="")
  print(path)
  
  # create zoo object and read prediction data
  predict <- read.csv(path)
  colnames(predict)[2] <- "P"
  colnames(predict)[3] <- "E"
  predict.zoo <- as.zoo(predict)
  predict.zoo <- read.zoo(predict, format = "%d/%m/%Y")
  
  # model
  output <- predict(LUE_mod, predict.zoo)
  
  # file path creator
  csv_path <- file.path("C:", "Users", "gy16e3m", "Downloads", paste(f,".csv", sep=""))
  
  # convert units to cumecs
  flows <- convertFlow(zoo(output), from = "mm/day", to =
                         "cumecs", area = 15320)
  
  # write to csv
  write.zoo(flows, file=csv_path, row.names=FALSE, col.names=TRUE, sep=",")
}

